@@ -1,17 +1,36 @@
 import copy
+import platform
+import os
 
 from nctoolkit.api import open_data
 from nctoolkit.cleanup import cleanup
 from nctoolkit.runthis import run_cdo, tidy_command
 from nctoolkit.show import nc_years
 from nctoolkit.temp_file import temp_file
-from nctoolkit.session import remove_safe, session_info
-from nctoolkit.utils import version_below
+from nctoolkit.session import remove_safe, session_info, nc_safe_par, nc_safe
+from nctoolkit.api import options
+from nctoolkit.runners import ann_anomaly
+
+if platform.system() == "Linux":
+    import multiprocessing as mp
+    #start = mp.get_start_method()
+    #if start != "spawn":
+    #    mp.set_start_method("spawn")
+    from multiprocessing import Manager
+
+    manager = Manager()
+else:
+    import multiprocess as mp
+    from multiprocess import Manager
+
+    manager = Manager()
 
 
-def annual_anomaly(self, baseline=None, metric="absolute", window=1, align = "right"):
+
+def annual_anomaly(self, baseline=None, metric="absolute", window=1, align="right"):
     """
-    Calculate annual anomalies for each variable based on a baseline period
+    annual_anomaly: Calculate annual anomalies for each variable based on a baseline period.
+
     The anomaly is derived by first calculating the climatological annual mean for the
     given baseline period. Annual means are then calculated for each year and the
     anomaly is calculated compared with the baseline mean. This will be calculated on a
@@ -88,7 +107,7 @@ def annual_anomaly(self, baseline=None, metric="absolute", window=1, align = "ri
     if not isinstance(baseline[0], int):
         raise TypeError("Provide a valid baseline")
     if not isinstance(baseline[1], int):
-        raise TypeError("Provide a vaid baseline")
+        raise TypeError("Provide a valid baseline")
     if baseline[1] < baseline[0]:
         raise ValueError("Second baseline year is before the first!")
 
@@ -99,63 +118,118 @@ def annual_anomaly(self, baseline=None, metric="absolute", window=1, align = "ri
     # This cannot possibly be threaded in cdo. Release it
 
     self.run()
+    for ff in self:
+        if len([yy for yy in baseline if yy not in nc_years(ff)]) > 0:
+            raise ValueError("Check that the years in baseline are in the dataset!")
 
     # calculate the anomalies for each file
     # this is not parallelized yet
     # list of new files created
-    new_files = []
+    new_files = manager.list()
     # list of new commands
-    new_commands = []
+    new_commands = manager.list()
 
-    for ff in self:
-        # create the target file
-        target = temp_file("nc")
+    # keep track of whether we started parallel, so it can be reset
+    start_parallel = session_info["parallel"]
 
-        # throw error if baseline is not valid
-        if len([yy for yy in baseline if yy not in nc_years(ff)]) > 0:
-            raise ValueError("Check that the years in baseline are in the dataset!")
-        # generate the cdo command
-        if metric == "absolute":
-            cdo_command = (
-                f"cdo -sub -runmean,{window} -yearmean {ff} -timmean "
-                f"-selyear,{baseline[0]}/{baseline[1]} {ff} {target}"
+    if start_parallel is False:
+        options(parallel =  True)
+
+    nc_safe_par_start = copy.deepcopy(nc_safe_par)
+    try:
+        # loop over the files and calculate the anomaly in parallel
+        cores = session_info["cores"]
+
+        target_list = []
+        results = dict()
+
+        pool = mp.get_context('fork').Pool(cores)
+        precision = copy.deepcopy(self._precision)
+        for ff in self:
+            results[ff] = pool.apply_async(
+                ann_anomaly,
+                [
+                    ff,
+                    baseline,
+                    metric,
+                    window,
+                    align,
+                    precision,
+                    new_files,
+                    new_commands,
+                    nc_safe_par,
+                ],
             )
+        for k, v in results.items():
+            out = v.get()
+            if "Check that the years in baseline are in the dataset!" in str(out):
+                raise ValueError("Check that the years in baseline are in the dataset!")
+
+        #pool.close()
+        #pool.join()
+
+        self.history += list(new_commands)
+        self._hold_history = copy.deepcopy(self.history)
+
+        if start_parallel is False:
+            #update_options({"parallel": False})
+            options(parallel = False)
+            while True:
+                for ff in nc_safe_par:
+                    if ff not in nc_safe:
+                        nc_safe.append(ff)
+                    nc_safe_par.remove(ff)
+                if len(nc_safe_par) == 0:
+                    break
+
+        self.current = list(new_files)
+
+        for ff in new_files:
+            remove_safe(ff)
+
+        if start_parallel is False:
+            for ff in list(set(new_files)):
+                while True:
+                    if ff in nc_safe:
+                        nc_safe.remove(ff)
+                    else:
+                        break
+                nc_safe.append(ff)
+
+        if start_parallel:
+            for ff in list(set(new_files)):
+                while True:
+                    if ff in nc_safe_par:
+                        nc_safe_par.remove(ff)
+                    else:
+                        break
+                nc_safe_par.append(ff)
+
+        cleanup()
+
+        self.disk_clean()
+    except BaseException as e:
+        if start_parallel is False:
+            #update_options({"parallel": False})
+            options(parallel = False)
+        if start_parallel:
+            for ff in nc_safe_par:
+                if ff not in nc_safe_par_start:
+                    nc_safe_par.remove(ff)
         else:
-            cdo_command = (
-                f"cdo -div -runmean,{window} -yearmean {ff} -timmean "
-                f"-selyear,{baseline[0]}/{baseline[1]} {ff} {target}"
-            )
+            for ff in nc_safe:
+                if ff not in nc_safe_par_start:
+                    nc_safe.remove(ff)
+            for ff in nc_safe_par:
+                nc_safe_par.remove(ff)
 
-        # run the command and save the temp file
-
-        cdo_command = tidy_command(cdo_command)
-        cdo_command = cdo_command.replace("cdo ", f"cdo --timestat_date {align} ").replace(
-                    "  ", " "
-                )
-
-        target = run_cdo(cdo_command, target, precision=self._precision)
-
-        # updae the new files and commands
-        new_files.append(target)
-        new_commands.append(cdo_command)
-
-    # update the history
-    self.history += new_commands
-    self._hold_history = copy.deepcopy(self.history)
-
-    self.current = new_files
-
-    for ff in new_files:
-        remove_safe(ff)
-
-    cleanup()
-
-    self.disk_clean()
+        raise e
 
 
 def monthly_anomaly(self, baseline=None):
     """
-    Calculate monthly anomalies based on a baseline period
+    monthly:anomaly: Calculate monthly anomalies based on a baseline period.
+
     The anomaly is derived by first calculating the climatological monthly mean for the
     given baseline period. Monthly means are then calculated for each year and the
     anomaly is calculated compared with the baseline mean. This is calculated separately
@@ -195,19 +269,10 @@ def monthly_anomaly(self, baseline=None):
 
     self.run()
 
-    if version_below(session_info["cdo"], "1.9.10"):
-        for ff in self:
-            ds = open_data(ff, checks=False)
-            if len(set(ds.contents.npoints)) > 1:
-                raise ValueError(
-                    "Variables have different grids. This causes problems with the CDO version installed. Select specific variables before monthly_anomaly or upgrade to CDO>=1.9.10. See npoints in dataset contents to identify differences in grids"
-                )
-
     new_files = []
     new_commands = []
 
     for ff in self:
-
         if len([yy for yy in baseline if yy not in nc_years(ff)]) > 0:
             raise ValueError("Check that the years in baseline are in the dataset!")
         # create the target file
