@@ -14,8 +14,7 @@ import platform
 
 from netCDF4 import Dataset
 
-from nctoolkit.cleanup import cleanup, clean_all, temp_check
-from nctoolkit.flatten import str_flatten
+from nctoolkit.cleanup import cleanup, temp_check
 from nctoolkit.runthis import run_cdo
 from nctoolkit.session import (
     nc_protected,
@@ -102,8 +101,16 @@ warnings.formatwarning = custom_formatwarning
 
 # set up the session info
 letters = string.ascii_lowercase
+try:
+    login = os.getlogin()
+except:
+    login = ""
 session_info["stamp"] = (
-    "nctoolkit" + "".join(random.choice(letters) for i in range(8)) + "nctoolkit"
+    "nctoolkit_"
+    + login
+    + "_"
+    + "".join(random.choice(letters) for i in range(8))
+    + "nctoolkit"
 )
 session_info["temp_dir"] = "/tmp/"
 session_info["user_dir"] = False
@@ -113,9 +120,26 @@ session_info["precision"] = None
 session_info["parallel"] = False
 session_info["progress"] = "on"
 session_info["checks"] = True
-session_info["coast"] = False
 session_info["user"] = ""
 
+# get the cdo methods
+
+read = subprocess.run(
+   "cdo --operators", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+).stdout
+
+cdo_methods = [x.split(" ")[0].replace("b'", "") for x in str(read).split("\\n")]
+
+cdo_methods = [mm for mm in cdo_methods if len(mm) > 0]
+
+cdo_methods.append("L")
+cdo_methods.append("reduce_dim")
+session_info["cdo_methods"] = cdo_methods
+
+
+session_info["coast"] = None 
+
+
 session_info["interactive"] = sys.__stdin__.isatty()
 
 
@@ -157,9 +181,15 @@ def update_options(kwargs):
             if not isinstance(kwargs[key], bool):
                 raise TypeError(f"{key} should be boolean")
 
+        if key == "coast":
+            if not isinstance(kwargs[key], bool):
+                raise TypeError(f"{key} should be boolean")
+            session_info["key"] = kwargs[key]
+
         if key == "checks":
             if not isinstance(kwargs[key], bool):
                 raise TypeError(f"{key} should be boolean")
+            find = False
 
         if not isinstance(kwargs[key], bool) and find:
             if key == "temp_dir":
@@ -232,7 +262,6 @@ def update_options(kwargs):
 
             # update safe-lists etc. if running in parallel
             if kwargs[key] and key == "parallel" and find:
-
                 if len(temp_dirs) > 0:
                     for ff in temp_dirs:
                         append_tempdirs(ff)
@@ -240,6 +269,7 @@ def update_options(kwargs):
                 if len(nc_safe) > 0:
                     for ff in nc_safe:
                         nc_safe_par.append(ff)
+                        nc_safe.remove(ff)
 
                 if len(nc_protected) > 0:
                     for ff in nc_protected:
@@ -247,7 +277,6 @@ def update_options(kwargs):
                         nc_protected.remove(ff)
 
             if (kwargs[key] is False) and key == "parallel" and find:
-
                 if len(temp_dirs_par) > 0:
                     for ff in temp_dirs_par:
                         append_tempdirs(ff)
@@ -266,20 +295,19 @@ def update_options(kwargs):
 def options(**kwargs):
     """
     Define session options.
-    Set the options in the session. Available options are thread_safe and lazy.
-    Set thread_safe = True if hdf5 was built to be thread safe.
-    Set lazy = False if you want methods to evaluate non-lazily
-    Set cores = n, if you want nctoolkit to process the individual files in multi-file datasets in parallel. Note this
-    only applies to multi-file datasets and will not improve performance with single files.
-    Set temp_dir = "/foo" if you want to change the temporary directory used by nctoolkit to save temporary files.
-    Set  = "/foo" if you want to change the temporary directory used by nctoolkit to save temporary files.
-    Set progress to "on" or "off" if you always or never want a progress bar to show when multi-file datasets are processed. This defaults to "auto", i.e.
-    nctoolkit will automatically decide whether to show a progress bar based on the size of the ensemble.
 
     Parameters
     ---------------
     **kwargs
         Define options using key, value pairs.
+        Set thread_safe = True if hdf5 was built to be thread safe.
+        Set lazy = False if you want methods to evaluate non-lazily
+        Set cores = n, if you want nctoolkit to process the individual files in multi-file datasets in parallel. Note this
+        only applies to multi-file datasets and will not improve performance with single files.
+        Set temp_dir = "/foo" if you want to change the temporary directory used by nctoolkit to save temporary files.
+        Set  = "/foo" if you want to change the temporary directory used by nctoolkit to save temporary files.
+        Set progress to "on" or "off" if you always or never want a progress bar to show when multi-file datasets are processed. This defaults to "auto", i.e.
+        nctoolkit will automatically decide whether to show a progress bar based on the size of the ensemble.
 
     Examples
     ------------
@@ -299,104 +327,9 @@ def options(**kwargs):
 
     """
 
-    valid_keys = [
-        "thread_safe",
-        "lazy",
-        "cores",
-        "precision",
-        "temp_dir",
-        "parallel",
-        "checks",
-        "progress",
-    ]
-
     update_options(kwargs)
     return None
 
-    for key in kwargs:
-        if key not in valid_keys:
-            raise AttributeError(key + " is not a valid option")
-
-        if key == "parallel" or key == "lazy" or key == "thread_safe":
-            if isinstance(kwargs[key], bool):
-                raise TypeError(f"{key} should be boolean")
-
-        if key == "checks":
-            if isinstance(kwargs[key], bool):
-                raise TypeError(f"{key} should be boolean")
-
-        if isinstance(kwargs[key], bool):
-            if key == "temp_dir":
-                if isinstance(kwargs[key], str):
-                    if os.path.exists(kwargs[key]) is False:
-                        raise ValueError("The temp_dir specified does not exist!")
-                    session_info[key] = os.path.abspath(kwargs[key])
-                    session_info["user_dir"] = True
-                return None
-            if key == "progress":
-                if kwargs[key] not in ["on", "off", "auto", "of"]:
-                    raise ValueError("progress must be one of 'on', 'off', 'auto'")
-
-                if kwargs[key] == "of":
-                    session_info[key] = "off"
-                else:
-                    session_info[key] = kwargs[key]
-                return None
-
-            if key == "cores":
-                if isinstance(kwargs[key], int):
-                    if kwargs[key] > mp.cpu_count():
-                        raise ValueError(
-                            str(kwargs[key])
-                            + " is greater than the number of system cores ("
-                            + str(mp.cpu_count())
-                            + ")"
-                        )
-                    session_info[key] = kwargs[key]
-                else:
-                    raise TypeError("cores must be an int")
-            else:
-                if key == "precision":
-                    if kwargs[key] not in ["I8", "I16", "I32", "F32", "F64"]:
-                        raise ValueError("precision supplied is not valid!")
-                    session_info[key] = kwargs[key]
-                else:
-                    raise ValueError(kwargs[key] + " is not valid session info!")
-        else:
-            session_info[key] = kwargs[key]
-
-            # update safe-lists etc. if running in parallel
-            if kwargs[key] and key == "parallel":
-
-                if len(temp_dirs) > 0:
-                    for ff in temp_dirs:
-                        append_tempdirs(ff)
-
-                if len(nc_safe) > 0:
-                    for ff in nc_safe:
-                        nc_safe_par.append(ff)
-
-                if len(nc_protected) > 0:
-                    for ff in nc_protected:
-                        nc_protected_par.append(ff)
-                        nc_protected.remove(ff)
-
-            if (kwargs[key] is False) and key == "parallel":
-
-                if len(temp_dirs_par) > 0:
-                    for ff in temp_dirs_par:
-                        append_tempdirs(ff)
-
-                if len(nc_safe_par) > 0:
-                    for ff in nc_safe_par:
-                        nc_safe.append(ff)
-                        nc_safe_par.remove(ff)
-
-                if len(nc_protected_par) > 0:
-                    for ff in nc_protected_par:
-                        nc_protected.append(ff)
-                        nc_protected_par.remove(ff)
-
 
 # if nctoolkitrc exists, we need to read the possible options from there...
 
@@ -495,7 +428,6 @@ def file_size(file_path):
 
 
 def from_xarray(ds):
-
     """
     Convert an xarray dataset to an nctoolkit dataset
     This will first save the xarray dataset as a temporary netCDF file.
@@ -520,7 +452,8 @@ def from_xarray(ds):
 
 def open_geotiff(x=[]):
     """
-    Read geotiff and convert to nctoolkit dataset
+    Open a geotiff and convert to a Dataset
+    This requires rioxarray to be installed.
 
     Parameters
     ---------------
@@ -530,7 +463,7 @@ def open_geotiff(x=[]):
 
     Returns
     ---------------
-    open_data : nctoolkit.DataSet
+    open_geotiff: nctoolkit.DataSet
     """
 
     if isinstance(x, str):
@@ -556,7 +489,7 @@ def open_geotiff(x=[]):
 
 def open_data(x=[], checks=True, **kwargs):
     """
-    Read netCDF data as a DataSet object
+    Read netCDF data as a Dataset object
 
     Parameters
     ---------------
@@ -567,7 +500,8 @@ def open_data(x=[], checks=True, **kwargs):
         By default an empty dataset is created, ie. using open_data() will create an empty
         dataset that can then be expanded using append.
     checks: boolean
-        Do you want basic checks to ensure cdo can read files? Default to True
+        Do you want basic checks to ensure cdo can read files? Default to True.
+        Setting to False can result in a minor speed up.
     **kwargs: kwargs
         Optional arguments for internal use by open_thredds and open_url.
 
@@ -620,6 +554,7 @@ def open_data(x=[], checks=True, **kwargs):
             source = kwargs[key]
 
     if isinstance(x, str) and thredds is False:
+        x = os.path.expanduser(x)
         if is_url(x) is False:
             x = glob.glob(x)
             if len(x) == 0:
@@ -640,6 +575,7 @@ def open_data(x=[], checks=True, **kwargs):
         x = [x]
 
     if isinstance(x, list):
+        x = [os.path.expanduser(y) for y in x]
         x = fix_files(x)
 
     if isinstance(x, list):
@@ -654,14 +590,11 @@ def open_data(x=[], checks=True, **kwargs):
         stop_time = min(file_stop, stop_time)
 
     if isinstance(x, str):
+        x = os.path.expanduser(x)
         if source != "file" or os.path.exists(x) is False:
-
             if is_url(x):
-
                 if thredds is False:
                     new_x = temp_file(".nc")
-                    print(f"Downloading {x}")
-                    print("\033[A                             \033[A")
 
                     if ftp_details is not None and x.startswith("ftp"):
                         user = ftp_details["user"]
@@ -698,7 +631,6 @@ def open_data(x=[], checks=True, **kwargs):
 
                     x = new_x
                 else:
-
                     if checks:
                         if wait is not None:
                             with time_limit(stop_time):
@@ -750,21 +682,15 @@ def open_data(x=[], checks=True, **kwargs):
 
     if isinstance(x, list):
         if source == "url":
-
             for ff in x:
-
                 if is_url(ff) is False:
                     raise ValueError(f"{x} is not a url")
 
             new_files = []
             for ff in x:
-
                 if is_url(ff):
-
                     if thredds is False:
                         new_x = temp_file(".nc")
-                        print(f"Downloading {ff}")
-                        print("\033[A                             \033[A")
 
                         if ftp_details is not None and x.startswith("ftp"):
                             user = ftp_details["user"]
@@ -804,7 +730,6 @@ def open_data(x=[], checks=True, **kwargs):
 
     if isinstance(x, list) and source != "url":
         if thredds is False:
-
             for ff in x:
                 if os.path.exists(ff) is False:
                     raise FileNotFoundError(f"{ff} does not exist!")
@@ -814,7 +739,6 @@ def open_data(x=[], checks=True, **kwargs):
 
             if len(x) > 1:
                 for ff in x:
-
                     if os.path.exists(ff) is False:
                         raise FileNotFoundError("Data set " + ff + " does not exist!")
 
@@ -861,7 +785,7 @@ def open_data(x=[], checks=True, **kwargs):
 
 def open_thredds(x=None, wait=None, checks=False):
     """
-    Read thredds data as a DataSet object
+    Read thredds data as a Dataset object
 
     Parameters
     ---------------
@@ -891,7 +815,7 @@ def open_thredds(x=None, wait=None, checks=False):
         raise TypeError("Please provide boolean for checks")
 
     if wait is not None:
-        if not instance(wait, (int, float)):
+        if not isinstance(wait, (int, float)):
             raise TypeError("Please provide an integer for wait!")
         if wait <= 0:
             raise ValueError("Please provide a positive value for wait!")
@@ -980,6 +904,12 @@ def merge(*datasets, match=["day", "year", "month"]):
         For example, if you want to ensure the datasets have the same years, then use
         match = ["year"].
     """
+    # A deprecation warning:
+    warnings.warn(
+        "This method is deprecated. Use the append and merge approach instead.",
+        DeprecationWarning,
+    )
+
     all_files = []
     for dataset in datasets:
         if not isinstance(dataset, DataSet):
@@ -1005,6 +935,11 @@ def cor_time(x=None, y=None):
         First dataset to use
     y: dataset
         Second dataset to use
+
+    Returns
+    -------------
+    cor_time: nctoolkit Dataset
+        A dataset containing the temporal correlation coefficient for each grid cell
     """
 
     if not isinstance(x, DataSet):
@@ -1075,6 +1010,11 @@ def cor_space(x=None, y=None):
     between two datasets. The datasets must either have the same variables or only
     have one variable.
 
+    Returns
+    -------------
+    cor_space: nctoolkit Dataset
+        A dataset containing the correlation coefficient for each time step.
+
     Parameters
     -------------
     x: dataset
@@ -1180,8 +1120,23 @@ class DataSet(object):
         # track number of held over commands
         self._ncommands = 0
 
-    def __getitem__(self, index):
+        # add a dictionary for caching attributes
+        self._atts = dict()
 
+        self._atts["variables"] = [None, -1]
+        self._atts["months"] = [None, -1]
+        self._atts["levels"] = [None, -1]
+        self._atts["times"] = [None, -1]
+        self._atts["calendar"] = [None, -1]
+        self._atts["size"] = [None, -1]
+        self._atts["ncformat"] = [None, -1]
+        self._atts["years"] = [None, -1]
+        self._atts["calendar"] = [None, -1]
+        self._atts["contents"] = [None, -1]
+
+
+
+    def __getitem__(self, index):
         return self.current[index]
 
     def __len__(self):
@@ -1193,6 +1148,9 @@ class DataSet(object):
         return
 
     def __repr__(self):
+        self.run()
+        if len(self) == 0:
+            return "Empty dataset"
         current = str(len(self))
 
         output = "<nctoolkit.DataSet>:\nNumber of files: " + current
@@ -1205,41 +1163,31 @@ class DataSet(object):
             output += "......"
         return output
 
-        # return(self.show_contents(min(12, len(self))))
-        #''    print(".......")
-
-        # return (
-        #    "<nctoolkit.DataSet>:\nFiles: "
-        #    + current
-        #    + "\n"
-        #    + "Variables: "
-        #    + str_flatten(variables)
-        # )
-
     @property
     def size(self):
         """The size of an object
         This will print the number of files, total size, and smallest and largest files
         in an DataSet object.
+
+        Returns
+        -------------
+        size: dict
+            A dictionary containing the number of files, total size, and smallest and
         """
+        self.run()
         all_sizes = []
 
-        smallest_file = ""
-        largest_file = ""
         min_size = 1e15
         max_size = -1
 
         for ff in self:
-
             all_sizes.append(file_size(ff))
 
             if file_size(ff) > max_size:
                 max_size = file_size(ff)
-                largest_file = ff
 
             if file_size(ff) < min_size:
                 min_size = file_size(ff)
-                smallest_file = ff
 
         min_size = convert_bytes(min_size)
         max_size = convert_bytes(max_size)
@@ -1257,9 +1205,20 @@ class DataSet(object):
     def calendar(self):
         """
         List calendars of dataset files
+
+        Returns
+        -------
+        list
+            List of calendars in the dataset files
+
         """
 
-        # return None
+        self.run()
+
+        n_commands = self._atts["calendar"][1]
+
+        if n_commands == len(self._hold_history):
+            return self._atts["calendar"][0]
 
         cals = []
         for ff in self:
@@ -1280,15 +1239,35 @@ class DataSet(object):
         cals = pd.concat(cals).reset_index(drop=True)
 
         if len(set(cals.calendar)) == 1:
+            self._atts["calendar"][0] = cals.calendar[0]
             return cals.calendar[0]
+
+        self._atts["calendar"][1] = len(self._hold_history)
+
+        self._atts["calendar"][0] = cals
+
         return cals
 
     @property
     def variables(self):
         """
         List variables contained in a dataset
+
+        Returns
+        -------
+        list
+            List of variables
+
         """
 
+        self.run()
+
+        n_commands = self._atts["variables"][1]
+
+        if n_commands == len(self._hold_history):
+            return self._atts["variables"][0]
+
+
         all_variables = []
         for ff in self:
             all_variables += nc_variables(ff)
@@ -1297,14 +1276,32 @@ class DataSet(object):
 
         all_variables.sort()
 
+        # populate the atts dictionary
+
+        self._atts["variables"][0] = all_variables
+        self._atts["variables"][1] = len(self._hold_history)
+
+
         return all_variables
 
     @property
     def months(self):
         """
         List months contained in a dataset
+
+        Returns
+        -------
+        list
+            List of months in the dataset
         """
 
+        self.run()
+
+        n_commands = self._atts["months"][1]
+
+        if n_commands == len(self._hold_history): 
+            return self._atts["months"][0]
+
         all_months = []
         for ff in self:
             all_months += nc_months(ff)
@@ -1313,14 +1310,32 @@ class DataSet(object):
 
         all_months.sort()
 
+        # populate the atts dictionary
+
+        self._atts["months"][0] = all_months
+        self._atts["months"][1] = len(self._hold_history)
+
         return all_months
 
     @property
     def levels(self):
         """
         List levels contained in a dataset
+
+        Returns
+        -------
+        list
+            List of levels in the dataset
+
         """
 
+        self.run()
+
+        n_commands = self._atts["levels"][1]
+
+        if n_commands == len(self._hold_history):
+            return self._atts["levels"][0]
+
         all_levels = []
         for ff in self:
             all_levels += nc_levels(ff)
@@ -1329,14 +1344,33 @@ class DataSet(object):
 
         all_levels.sort()
 
+        # populate the atts dictionary
+
+        self._atts["levels"][0] = all_levels
+        self._atts["levels"][1] = len(self._hold_history)
+
+
         return all_levels
 
     @property
     def times(self):
         """
         List times contained in a dataset
+
+        Returns
+        -------
+        list
+            List of times in the dataset
         """
 
+        self.run()
+
+        n_commands = self._atts["times"][1]
+
+        if n_commands == len(self._hold_history):
+            return self._atts["times"][0]
+
+
         all_times = []
         for ff in self:
             all_times += nc_times(ff)
@@ -1345,14 +1379,27 @@ class DataSet(object):
 
         all_times.sort()
 
+        # populate the atts dictionary
+
+        self._atts["times"][0] = all_times
+        self._atts["times"][1] = len(self._hold_history)
+
+
         return all_times
 
     @property
     def ncformat(self):
         """
         List formats of files contained in a dataset
+
+        Return
+        ------
+        list
+            List of netCDF formats in the dataset
         """
 
+        self.run()
+
         all_formats = []
         for ff in self:
             all_formats += nc_format(ff)
@@ -1367,8 +1414,21 @@ class DataSet(object):
     def years(self):
         """
         List years contained in a dataset
+
+        Returns
+        -------
+        list
+            List of years in the dataset
+
         """
 
+        self.run()
+
+        n_commands = self._atts["years"][1]
+
+        if n_commands == len(self._hold_history):
+            return self._atts["years"][0]
+
         all_years = []
         for ff in self:
             all_years += nc_years(ff)
@@ -1377,6 +1437,13 @@ class DataSet(object):
 
         all_years.sort()
 
+        # populate the atts dictionary
+
+        self._atts["years"][0] = all_years
+
+        self._atts["years"][1] = len(self._hold_history)
+
+
         return all_years
 
     def show_contents(self, n=None):
@@ -1386,7 +1453,6 @@ class DataSet(object):
         """
 
         try:
-
             if n is None:
                 n = len(self)
 
@@ -1399,7 +1465,6 @@ class DataSet(object):
                     use_names = False
 
             for ff in self[0:n]:
-
                 dataset = Dataset(ff)
 
                 out = subprocess.run(
@@ -1508,7 +1573,6 @@ class DataSet(object):
                 )
 
                 try:
-
                     times = []
 
                     ds = xr.open_dataset(ff, decode_times=False)
@@ -1605,14 +1669,35 @@ class DataSet(object):
         """
         Detailed list of variables contained in a dataset.
         This will only display the variables in the first file of an ensemble.
+
+        Returns
+        -------
+        pandas.DataFrame
+            A pandas dataframe containing the variables, number of points, number of levels, long name, units, and data type of each variable in the dataset.
         """
 
-        return self.show_contents()
+        self.run()
+
+        n_commands = self._atts["contents"][1]
+
+        if n_commands == len(self._hold_history):
+            return self._atts["contents"][0]
+        
+        df = self.show_contents()
+        self._atts["contents"][0] = df
+        self._atts["contents"][1] = len(self._hold_history)
+
+        return df
 
     @property
     def start(self):
         """
         The starting file or files of the DataSet object
+
+        Returns
+        -------
+        list
+            A list of the starting files in the DataSet object
         """
         return self._start
 
@@ -1627,6 +1712,11 @@ class DataSet(object):
     def current(self):
         """
         The current file or files in the DataSet object
+
+        Returns
+        -------
+        list
+            A list of the current files in the DataSet object
         """
         return self._current
 
@@ -1648,6 +1738,11 @@ class DataSet(object):
     def history(self):
         """
         The history of operations on the DataSet
+
+        Returns
+        -------
+        list
+            A list of the operations performed on the DataSet
         """
         return self._history
 
@@ -1659,6 +1754,7 @@ class DataSet(object):
         """
         Make a deep copy of an DataSet object.
         Note: This will not make disk copies of the temporary files underlying datasets, so it will be disk-space efficient.
+
         Returns
         ---------------
         copy: nctoolkit DataSet
@@ -1768,6 +1864,7 @@ class DataSet(object):
 
     from nctoolkit.ensembles import ensemble_mean
     from nctoolkit.ensembles import ensemble_max
+    from nctoolkit.ensembles import ensemble_median
     from nctoolkit.ensembles import ensemble_min
     from nctoolkit.ensembles import ensemble_range
     from nctoolkit.ensembles import ensemble_stdev
@@ -1778,6 +1875,7 @@ class DataSet(object):
 
     from nctoolkit.esoteric import assign_coords
     from nctoolkit.esoteric import fix_nemo_ersem_grid
+    from nctoolkit.esoteric import fix_amm7_grid
     from nctoolkit.esoteric import set_gridtype
     from nctoolkit.esoteric import no_leaps
     from nctoolkit.esoteric import as_double
@@ -1805,6 +1903,8 @@ class DataSet(object):
     from nctoolkit.inttime import time_interp
     from nctoolkit.inttime import timestep_interp
 
+    from nctoolkit.invert import invert
+
     from nctoolkit.masking import mask_box
 
     from nctoolkit.mergers import collect
@@ -1821,9 +1921,10 @@ class DataSet(object):
 
     from nctoolkit.phenology import phenology
 
-    # from nctoolkit.phenology import initiation
 
     from nctoolkit.plot import plot
+    from nctoolkit.static_plot import pub_plot
+    from nctoolkit.static_plot import quiver_plot
 
     from nctoolkit.reduce import reduce_dims
 
@@ -1848,6 +1949,8 @@ class DataSet(object):
 
     from nctoolkit.subset import subset
 
+    from nctoolkit.set import set
+
     from nctoolkit.setters import set_date
     from nctoolkit.setters import set_year
     from nctoolkit.setters import set_day
@@ -1904,7 +2007,7 @@ class DataSet(object):
     from nctoolkit.verticals import bottom
     from nctoolkit.verticals import bottom_mask
     from nctoolkit.verticals import surface_mask
-    from nctoolkit.verticals import invert_levels
+    from nctoolkit.verticals import generate_level_bounds
 
     from nctoolkit.zip import zip
 
@@ -1914,5 +2017,5 @@ class DataSet(object):
     from nctoolkit.zonals import zonal_range
     from nctoolkit.zonals import zonal_sum
 
-    # Deprecated methods
-    # from nctoolkit.deprecated import set_missing
+    # deprecated methods
+    from nctoolkit.deprecated import invert_levels
