@@ -6,7 +6,7 @@ import nctoolkit.api as api
 
 def append(self, x=None):
     """
-    Add new file(s) to a dataset.
+    append: Add new file(s) to a dataset.
 
     Parameters
     -------------
@@ -50,9 +50,7 @@ def append(self, x=None):
 
     for ff in x:
         if ff in check_list:
-            warnings.warn(
-                "You are trying to add a file that is already in the dataset"
-            )
+            warnings.warn("You are trying to add a file that is already in the dataset")
 
     for ff in x:
         if os.path.exists(ff) is False:
@@ -65,7 +63,7 @@ def append(self, x=None):
 
 def remove(self, x=None):
     """
-    Remove file(s) from a dataset
+    remove: Remove file(s) from a dataset
 
     Parameters
     -------------
@@ -93,3 +91,15 @@ def remove(self, x=None):
     for ff in x:
         self.current.remove(ff)
         remove_safe(ff)
+
+    # if files have been removed, we need to reset the attributes
+    self._atts["variables"] = [None, -1]
+    self._atts["months"] = [None, -1]
+    self._atts["levels"] = [None, -1]
+    self._atts["times"] = [None, -1]
+    self._atts["calendar"] = [None, -1]
+    self._atts["size"] = [None, -1]
+    self._atts["ncformat"] = [None, -1]
+    self._atts["years"] = [None, -1]
+    self._atts["calendar"] = [None, -1]
+    self._atts["contents"] = [None, -1]
