@@ -1,4 +1,9 @@
-from nctoolkit.runthis import run_this
+def is_integer(x):
+    try:
+        y = int(str(x))
+        return True
+    except:
+        return False
 
 
 def fix_expr(expression):
@@ -10,14 +15,20 @@ def fix_expr(expression):
 
     # equal constant case
     if expression.startswith("==-"):
-        if expression.replace("==-", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("==-", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("==", "eqc")
         expression = expression.replace("eqc", "eqc,")
         return expression
     # equal constant case
     if expression.startswith("=="):
-        if expression.replace("==", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("==", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("==", "eqc")
         expression = expression.replace("eqc", "eqc,")
@@ -25,14 +36,20 @@ def fix_expr(expression):
 
     # not equal constant case
     if expression.startswith("!=-"):
-        if expression.replace("!=-", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("!=-", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("!=", "nec")
         expression = expression.replace("nec", "nec,")
         return expression
     # not equal constant case
     if expression.startswith("!="):
-        if expression.replace("!=", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("!=", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("!=", "nec")
         expression = expression.replace("nec", "nec,")
@@ -40,7 +57,10 @@ def fix_expr(expression):
 
     # less than or equal to constant case
     if expression.startswith("<=-"):
-        if expression.replace("<=-", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("<=-", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("<=", "lec")
         expression = expression.replace("lec", "lec,")
@@ -48,7 +68,10 @@ def fix_expr(expression):
 
     # less than or equal to constant case
     if expression.startswith("<="):
-        if expression.replace("<=", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("<=", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("<=", "lec")
         expression = expression.replace("lec", "lec,")
@@ -56,14 +79,20 @@ def fix_expr(expression):
 
     # less than or equal to constant case
     if expression.startswith("<-"):
-        if expression.replace("<-", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("<-", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("<", "ltc")
         expression = expression.replace("ltc", "ltc,")
         return expression
     # less than or equal to constant case
     if expression.startswith("<"):
-        if expression.replace("<", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace("<", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace("<", "ltc")
         expression = expression.replace("ltc", "ltc,")
@@ -71,14 +100,20 @@ def fix_expr(expression):
 
     # greater than or equal to constant case
     if expression.startswith(">=-"):
-        if expression.replace(">=-", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace(">=-", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace(">=", "gec")
         expression = expression.replace("gec", "gec,")
         return expression
 
     if expression.startswith(">="):
-        if expression.replace(">=", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace(">=", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace(">=", "gec")
         expression = expression.replace("gec", "gec,")
@@ -86,14 +121,20 @@ def fix_expr(expression):
 
     # greater than or equal to constant case
     if expression.startswith(">-"):
-        if expression.replace(">-", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace(">-", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace(">", "gtc")
         expression = expression.replace("gtc", "gtc,")
         return expression
 
     if expression.startswith(">"):
-        if expression.replace(">", "").replace(".", "", 1).isdigit() is False:
+        if (
+            expression.replace(">", "").replace(".", "", 1).replace("e", "").isdigit()
+            is False
+        ):
             raise ValueError(expression + " is not valid!")
         expression = expression.replace(">", "gtc")
         expression = expression.replace("gtc", "gtc,")
@@ -118,12 +159,20 @@ def compare(self, expression=None):
 
     >>> ds.compare(">0")
 
+    This could also be done using standard python syntax:
+
+    >>> ds > 0
+
     This will be calculcated for each time step.
 
     If you wanted to identify grid cells with negative values, you would do this
 
     >>> ds.compare("<0")
 
+    This could also be done using standard python syntax:
+
+    >>> ds < 0
+
 
     """
 
@@ -134,16 +183,18 @@ def compare(self, expression=None):
         raise TypeError("Expression supplied is not str")
 
     expression = fix_expr(expression)
-    cdo_command = f"cdo -{expression}"
-    run_this(cdo_command, self, output="ensemble")
+    cdo_command = f"-{expression}"
+    self.cdo_command(cdo_command, ensemble=False)
 
 
 def __eq__(self, x):
-
     if not isinstance(x, str):
         try:
             test = float(x)
-            self.compare(f"=={x}")
+            x_new = x
+            if not is_integer(x):
+                x_new = str(float(x)).replace("+", "")
+            self.compare(f"=={x_new}")
         except:
             self.eq(x)
     else:
@@ -151,33 +202,41 @@ def __eq__(self, x):
 
 
 def __gt__(self, x):
-
     if not isinstance(x, str):
         try:
             test = float(x)
-            self.compare(f">{x}")
+            x_new = x
+            if not is_integer(x):
+                x_new = str(float(x)).replace("+", "")
+            self.compare(f">{x_new}")
         except:
             self.gt(x)
     else:
         self.gt(x)
 
-def __lt__(self, x):
 
+def __lt__(self, x):
     if not isinstance(x, str):
         try:
             test = float(x)
-            self.compare(f"<{x}")
+            x_new = x
+            if not is_integer(x):
+                x_new = str(float(x)).replace("+", "")
+            self.compare(f"<{x_new}")
         except:
             self.lt(x)
     else:
         self.lt(x)
 
-def __le__(self, x):
 
+def __le__(self, x):
     if not isinstance(x, str):
         try:
             test = float(x)
-            self.compare(f"<={x}")
+            x_new = x
+            if not is_integer(x):
+                x_new = str(float(x)).replace("+", "")
+            self.compare(f"<={x_new}")
         except:
             self.le(x)
     else:
@@ -185,25 +244,28 @@ def __le__(self, x):
 
 
 def __ge__(self, x):
-
     if not isinstance(x, str):
         try:
             test = float(x)
-            self.compare(f">={x}")
+            x_new = x
+            if not is_integer(x):
+                x_new = str(float(x)).replace("+", "")
+            self.compare(f">={x_new}")
         except:
             self.ge(x)
     else:
         self.ge(x)
 
-def __ne__(self, x):
 
+def __ne__(self, x):
     if not isinstance(x, str):
         try:
             test = float(x)
-            self.compare(f"!={x}")
+            x_new = x
+            if not is_integer(x):
+                x_new = str(float(x)).replace("+", "")
+            self.compare(f"!={x_new}")
         except:
             self.ne(x)
     else:
         self.ne(x)
-
-
