@@ -1,5 +1,5 @@
 from nctoolkit.flatten import str_flatten
-from nctoolkit.runthis import run_this
+
 
 def fix_ind(x):
     if x < 0:
@@ -7,9 +7,11 @@ def fix_ind(x):
     else:
         return x + 1
 
+
 def drop(self, **kwargs):
     """
-    Remove variables
+    drop: Remove variables, days, months, years or time steps from a dataset
+
     This will remove stated variables from files in the dataset.
 
     Parameters
@@ -19,6 +21,7 @@ def drop(self, **kwargs):
 
         Note: this uses partial matches. So years, month, variable etc. will also work
 
+
     Each kwarg works as follows:
 
     var: str or list
@@ -64,7 +67,7 @@ def drop(self, **kwargs):
         if key_check > 1 or key_check == 0:
             raise ValueError("Drop term is invalid or ambiguous")
 
-    temporal_command = "cdo -delete"
+    temporal_command = "-delete"
 
     for key in kwargs:
         if "time" in key:
@@ -77,13 +80,12 @@ def drop(self, **kwargs):
                 if not isinstance(vv, int):
                     raise TypeError(f"{vv} is not an int")
 
-
             vars = [fix_ind(x) for x in vars]
             vars = str_flatten(vars, ",")
 
             # create the cdo command and run it
-            cdo_command = f"cdo -delete,timestep={vars}"
-            run_this(cdo_command, self, output="ensemble")
+            cdo_command = f"-delete,timestep={vars}"
+            self.cdo_command(cdo_command, ensemble=False)
 
         if "var" in key:
             vars = kwargs[key]
@@ -98,8 +100,8 @@ def drop(self, **kwargs):
             vars = str_flatten(vars, ",")
 
             # create the cdo command and run it
-            cdo_command = f"cdo -delete,name={vars}"
-            run_this(cdo_command, self, output="ensemble")
+            cdo_command = f"-delete,name={vars}"
+            self.cdo_command(cdo_command, ensemble=False)
 
         if ("mon" in key) or ("day" in key) or ("year" in key):
             vars = kwargs[key]
@@ -124,5 +126,5 @@ def drop(self, **kwargs):
             if "year" in key:
                 temporal_command = temporal_command + f",year={vars}"
 
-    if len(temporal_command) > len("cdo -delete"):
-        run_this(temporal_command, self, output="ensemble")
+    if len(temporal_command) > len("-delete"):
+        self.cdo_command(temporal_command, ensemble=False)
