@@ -3,22 +3,31 @@ import warnings
 
 from nctoolkit.cleanup import cleanup
 from nctoolkit.flatten import str_flatten
-from nctoolkit.runthis import run_this, run_nco
+from nctoolkit.runthis import run_nco
 from nctoolkit.temp_file import temp_file
 from nctoolkit.session import get_safe, remove_safe
 
 
 def ensemble_percentile(self, p=None):
     """
-    Calculate an ensemble percentile
-    This will calculate the percentles for each time step in the files.
+    ensemble_percentile: Calculate an ensemble percentile.
+
+    This will calculate the percentiles for each time step in the files.
     For example, if you had an ensemble of files where each file included
     12 months of data, it would calculate the percentile for each month.
+    This operates on a grid cell by grid cell basis.
 
     Parameters
     -------------
     p : float or int
         percentile to calculate. 0<=p<=100.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble 90th percentile as follows:
+
+    >>> ds.ensemble_percentile(p=90)
+
     """
 
     # make sure p is a number
@@ -41,8 +50,8 @@ def ensemble_percentile(self, p=None):
         warnings.warn(message="There is only one file in the dataset")
 
     # create the cdo command and run it
-    cdo_command = f"cdo --sortname -enspctl,{p}"
-    run_this(cdo_command, self, output="one")
+    cdo_command = f"--sortname -enspctl,{p}"
+    self.cdo_command(cdo_command, ensemble=True)
 
     # set the _merged attribute to True
     self._merged = True
@@ -68,10 +77,9 @@ def ensemble_nco(self, method, ignore_time=False):
     # run the call
     target = run_nco(nco_command, target)
 
+    remove_target = False
     if target in get_safe():
         remove_target = True
-    else:
-        remove_target = False
 
     # add the call to the history and tempfile to nc_safe
     self.history.append(nco_command)
@@ -90,10 +98,19 @@ def ensemble_nco(self, method, ignore_time=False):
 
 def ensemble_stdev(self):
     """
-    Calculate an ensemble standard deviation
+    ensemble_stdev: Calculate an ensemble standard deviation
 
     The ensemble standard deviation is calculated for each time steps; for example, if the ensemble is made up of
     monthly files the standard deviation for each month will be calculated.
+    This operates on a grid cell by grid cell basis.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble standard deviation as follows:
+
+    >>> ds.ensemble_stdev()
+
+
     """
 
     self.run()
@@ -101,19 +118,25 @@ def ensemble_stdev(self):
     if len(self) == 1:
         warnings.warn(message="There is only one file in the dataset")
 
-    cdo_command = "cdo --sortname -ensstd"
+    cdo_command = "--sortname -ensstd"
 
-    run_this(cdo_command, self)
-
-    self._merged = True
+    self.cdo_command(cdo_command, ensemble=True)
 
 
 def ensemble_var(self):
     """
-    Calculate an ensemble variance
+    ensemble_var: Calculate an ensemble variance
 
     The ensemble variance is calculated for each time steps; for example, if the ensemble is made up of
     monthly files the standard deviation for each month will be calculated.
+    This operates on a grid cell by grid cell basis.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble variance as follows:
+
+    >>> ds.ensemble_var()
+
     """
 
     self.run()
@@ -121,16 +144,42 @@ def ensemble_var(self):
     if len(self) == 1:
         warnings.warn(message="There is only one file in the dataset")
 
-    cdo_command = "cdo --sortname -ensvar"
+    cdo_command = "--sortname -ensvar"
 
-    run_this(cdo_command, self)
+    self.cdo_command(cdo_command, ensemble=True)
 
-    self._merged = True
+def ensemble_median(self):
+    """
+    ensemble_median: Calculate an ensemble median
+
+    The ensemble median is calculated for each time steps; for example, if the ensemble is made up of
+    monthly files the median for each month will be calculated.
+    This operates on a grid cell by grid cell basis.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble median as follows:
+
+    >>> ds.ensemble_median()
+
+    """
+
+    self.run()
+
+    if len(self) == 1:
+        warnings.warn(message="There is only one file in the dataset, so ensemble median cannot be calculated.")
+        return None
+
+    cdo_command = "--sortname -enspctl,50"
+
+    self.cdo_command(cdo_command, ensemble=True)
 
 
 def ensemble_max(self, nco=False, ignore_time=False):
     """
-    Calculate an ensemble maximum
+    ensemble_max: Calculate an ensemble maximum
+
+    This operates on a grid cell by grid cell basis.
 
     Parameters
     -------------
@@ -141,6 +190,14 @@ def ensemble_max(self, nco=False, ignore_time=False):
         If True the max is calculated over all time steps. If False, the ensemble max
         is calculated for each time steps; for example, if the ensemble is made up of
         monthly files the max for each month will be calculated.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble max as follows:
+
+    >>> ds.ensemble_max()
+
+
     """
 
     if nco is False:
@@ -150,13 +207,11 @@ def ensemble_max(self, nco=False, ignore_time=False):
             warnings.warn(message="There is only one file in the dataset")
 
         if ignore_time is False:
-            cdo_command = "cdo --sortname -ensmax"
+            cdo_command = "--sortname -ensmax"
         else:
-            cdo_command = "cdo -timmax --sortname -ensmax"
+            cdo_command = "-timmax --sortname -ensmax"
 
-        run_this(cdo_command, self)
-
-        self._merged = True
+        self.cdo_command(cdo_command, ensemble=True)
 
         return None
 
@@ -165,7 +220,9 @@ def ensemble_max(self, nco=False, ignore_time=False):
 
 def ensemble_min(self, nco=False, ignore_time=False):
     """
-    Calculate an ensemble min
+    ensemble_min: Calculate an ensemble minimum.
+
+    This operates on a grid cell by grid cell basis.
 
     Parameters
     -------------
@@ -176,6 +233,18 @@ def ensemble_min(self, nco=False, ignore_time=False):
         If True the min is calculated over all time steps. If False, the ensemble min is
         calculated for each time steps; for example, if the ensemble is made up of
         monthly files the min for each month will be calculated.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble min as follows:
+
+    >>> ds.ensemble_min()
+
+
+
+
+
+
     """
     if len(self) == 1:
         warnings.warn(message="There is only one file in the dataset")
@@ -184,13 +253,11 @@ def ensemble_min(self, nco=False, ignore_time=False):
         self.run()
 
         if ignore_time is False:
-            cdo_command = "cdo --sortname -ensmin"
+            cdo_command = "--sortname -ensmin"
         else:
-            cdo_command = "cdo -timmin --sortname -ensmin"
+            cdo_command = "-timmin --sortname -ensmin"
 
-        run_this(cdo_command, self)
-
-        self._merged = True
+        self.cdo_command(cdo_command, ensemble=True)
 
         return None
 
@@ -199,7 +266,9 @@ def ensemble_min(self, nco=False, ignore_time=False):
 
 def ensemble_mean(self, nco=False, ignore_time=False):
     """
-    Calculate an ensemble mean
+    ensemble_mean: Calculate an ensemble mean
+
+    This operates on a grid cell by grid cell basis.
 
     Parameters
     -------------
@@ -210,23 +279,31 @@ def ensemble_mean(self, nco=False, ignore_time=False):
         If True the mean is calculated over all time steps. If False, the ensemble mean
         is calculated for each time steps; for example, if the ensemble is made up of
         monthly files the mean for each month will be calculated.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble mean as follows:
+    ds.ensemble_mean()
+
+    If you had an ensemble of files that covered different time steps and want to calculate the mean over all time steps, you would do the following:
+
+    ds.ensemble_mean(ignore_time=True)
+
+
     """
 
     if nco is False:
-
         self.run()
 
         if len(self) == 1:
             warnings.warn(message="There is only one file in the dataset")
 
         if ignore_time is False:
-            cdo_command = "cdo --sortname -ensmean"
+            cdo_command = "--sortname -ensmean"
         else:
-            cdo_command = "cdo -timmean --sortname -ensmean"
+            cdo_command = "-timmean --sortname -ensmean"
 
-        run_this(cdo_command, self)
-
-        self._merged = True
+        self.cdo_command(cdo_command, ensemble=True)
 
         return None
 
@@ -235,35 +312,50 @@ def ensemble_mean(self, nco=False, ignore_time=False):
 
 def ensemble_range(self):
     """
-    Calculate an ensemble range
+    ensemble_range: Calculate an ensemble range
+
     The range is calculated for each time step; for example, if each file in the
     ensemble has 12 months of data the statistic will be calculated for each month.
+
+    This operates on a grid cell by grid cell basis.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble range as follows:
+
+    >>> ds.ensemble_range()
     """
     self.run()
 
     if len(self) == 1:
         warnings.warn(message="There is only one file in the dataset")
 
-    cdo_command = "cdo --sortname -ensrange"
+    cdo_command = "--sortname -ensrange"
 
-    run_this(cdo_command, self)
-
-    self._merged = True
+    self.cdo_command(cdo_command, ensemble=True)
 
 
 def ensemble_sum(self):
     """
-    Calculate an ensemble sum
+    ensemble_sum: Calculate an ensemble sum
+
     The sum is calculated for each time step; for example, if each file in the
     ensemble has 12 months of data the statistic will be calculated for each month.
+
+    This operates on a grid cell by grid cell basis.
+
+    Examples
+    -------------
+    If you had an ensemble of climate models with data covering the same time steps, you would calculate the ensemble sum as follows:
+
+    >>> ds.ensemble_sum()
+
     """
     self.run()
 
     if len(self) == 1:
         warnings.warn(message="There is only one file in the dataset")
 
-    cdo_command = "cdo --sortname -enssum"
+    cdo_command = "--sortname -enssum"
 
-    run_this(cdo_command, self)
-
-    self._merged = True
+    self.cdo_command(cdo_command, ensemble=True)
