@@ -5,9 +5,78 @@ import xarray as xr
 
 from .temp_file import temp_file
 from .cleanup import cleanup
-from .runthis import run_this
 from .runthis import run_nco
 from .api import open_data
+import warnings
+
+
+def fix_amm7_grid(self):
+    """
+    A quick hack to change the grid file in AMM7 North West European shelf Nemo grids.
+
+    """
+
+    ds = open_data(self[0])
+    # check the number of grid cells
+    if max(ds.contents.npoints) != 111375:
+        warnings.warn("This is not a NEMO ERSEM grid")
+        return None
+
+    lon_name = [x for x in ds.to_xarray().coords if "lon" in x][0]
+    lat_name = [x for x in ds.to_xarray().coords if "lat" in x][0]
+    lons = ds.to_xarray()[lon_name].values.flatten()
+    if (
+        len(set(list(pd.DataFrame({"lon": lons}).groupby("lon").size().sort_values())))
+        == 1
+    ):
+        raise ValueError("There appears to be nothing to fix!")
+    lons = list(set(lons))
+
+    for i in range(1, len(lons)):
+        if np.round(lons[i] - lons[i - 1], 5) == np.round(lons[i + 1] - lons[i], 5):
+            break
+    xinc = np.round(lons[i] - lons[i - 1], 5)
+
+    lats = ds.to_xarray()[lat_name].values.flatten()
+    lats = list(set(lats))
+
+    for i in range(1, len(lats)):
+        if np.round(lats[i] - lats[i - 1], 5) == np.round(lats[i + 1] - lats[i], 5):
+            break
+    yinc = np.round(lats[i] - lats[i - 1], 5)
+    grid_file = temp_file().replace(".", "")
+    ysize = ds.to_xarray()[lon_name].shape[0]
+    xsize = ds.to_xarray()[lat_name].shape[1]
+    xfirst = min(lons)
+    lats = ds.to_xarray()[lat_name].values.flatten()
+    yfirst = min(
+        pd.DataFrame({"lat": lats})
+        .groupby("lat")
+        .size()
+        .sort_values()
+        .reset_index()["lat"][0:-1]
+    )
+    with open(grid_file, "a") as file:
+        file.write("#\n")
+        file.write("# gridID 1\n")
+        file.write("#\n")
+        file.write("gridtype = lonlat\n")
+        file.write(f"gridsize = {xsize*ysize}\n")
+        file.write(f"xsize = {xsize}\n")
+        file.write(f"ysize = {ysize}\n")
+        file.write("xname = lon\n")
+        file.write("xlongname= longitude\n")
+        file.write("xunits= degrees_east\n")
+        file.write("yname = lat\n")
+        file.write("ylongname= latitude\n")
+        file.write("yunits= degrees_north\n")
+        file.write(f"xfirst= {xfirst}\n")
+        file.write(f"xinc= {xinc}\n")
+        file.write(f"yfirst= {yfirst}\n")
+        file.write(f"yinc= {yinc}\n")
+    self.cdo_command(f"-setgrid,{grid_file}")
+    self.run()
+
 
 def fix_nemo_ersem_grid(self):
     """
@@ -16,51 +85,64 @@ def fix_nemo_ersem_grid(self):
     """
 
     ds = open_data(self[0])
+    # check the number of grid cells
+    if max(ds.contents.npoints) != 111375:
+        warnings.warn("This is not a NEMO ERSEM grid")
+        return None
 
     lon_name = [x for x in ds.to_xarray().coords if "lon" in x][0]
     lat_name = [x for x in ds.to_xarray().coords if "lat" in x][0]
     lons = ds.to_xarray()[lon_name].values.flatten()
-    if len(set(list(pd.DataFrame({"lon":lons}).groupby("lon").size().sort_values()))) == 1:
+    if (
+        len(set(list(pd.DataFrame({"lon": lons}).groupby("lon").size().sort_values())))
+        == 1
+    ):
         raise ValueError("There appears to be nothing to fix!")
-    lons =  list(set(lons))
-    
+    lons = list(set(lons))
+
     for i in range(1, len(lons)):
-        if np.round(lons[i] - lons[i-1], 5) == np.round(lons[i+1] - lons[i], 5):
+        if np.round(lons[i] - lons[i - 1], 5) == np.round(lons[i + 1] - lons[i], 5):
             break
-    xinc = np.round(lons[i] - lons[i-1], 5)
-    
+    xinc = np.round(lons[i] - lons[i - 1], 5)
+
     lats = ds.to_xarray()[lat_name].values.flatten()
-    lats =  list(set(lats))
-    
+    lats = list(set(lats))
+
     for i in range(1, len(lats)):
-        if np.round(lats[i] - lats[i-1], 5) == np.round(lats[i+1] - lats[i], 5):
+        if np.round(lats[i] - lats[i - 1], 5) == np.round(lats[i + 1] - lats[i], 5):
             break
-    yinc = np.round(lats[i] - lats[i-1], 5)
+    yinc = np.round(lats[i] - lats[i - 1], 5)
     grid_file = temp_file().replace(".", "")
     ysize = ds.to_xarray()[lon_name].shape[0]
     xsize = ds.to_xarray()[lat_name].shape[1]
     xfirst = min(lons)
     lats = ds.to_xarray()[lat_name].values.flatten()
-    yfirst = min(pd.DataFrame({"lat":lats}).groupby("lat").size().sort_values().reset_index()["lat"][0:-1])
-    with open(grid_file, 'a') as file:
-        file.write('#\n')
-        file.write('# gridID 1\n')
-        file.write('#\n')
-        file.write('gridtype = lonlat\n')
-        file.write(f'gridsize = {xsize*ysize}\n')
-        file.write(f'xsize = {xsize}\n')
-        file.write(f'ysize = {ysize}\n')
-        file.write('xname = lon\n')
-        file.write('xlongname= longitude\n')
-        file.write('xunits= degrees_east\n')
-        file.write('yname = lat\n')
-        file.write('ylongname= latitude\n')
-        file.write('yunits= degrees_north\n')
-        file.write(f'xfirst= {xfirst}\n')
-        file.write(f'xinc= {xinc}\n')
-        file.write(f'yfirst= {yfirst}\n')
-        file.write(f'yinc= {yinc}\n')
-    self.cdo_command(f"setgrid,{grid_file}")
+    yfirst = min(
+        pd.DataFrame({"lat": lats})
+        .groupby("lat")
+        .size()
+        .sort_values()
+        .reset_index()["lat"][0:-1]
+    )
+    with open(grid_file, "a") as file:
+        file.write("#\n")
+        file.write("# gridID 1\n")
+        file.write("#\n")
+        file.write("gridtype = lonlat\n")
+        file.write(f"gridsize = {xsize*ysize}\n")
+        file.write(f"xsize = {xsize}\n")
+        file.write(f"ysize = {ysize}\n")
+        file.write("xname = lon\n")
+        file.write("xlongname= longitude\n")
+        file.write("xunits= degrees_east\n")
+        file.write("yname = lat\n")
+        file.write("ylongname= latitude\n")
+        file.write("yunits= degrees_north\n")
+        file.write(f"xfirst= {xfirst}\n")
+        file.write(f"xinc= {xinc}\n")
+        file.write(f"yfirst= {yfirst}\n")
+        file.write(f"yinc= {yinc}\n")
+    self.cdo_command(f"-setgrid,{grid_file}")
     self.run()
 
 
@@ -69,11 +151,16 @@ def no_leaps(self):
     Remove leap years.
     This uses an undocumented CDO feature to remove Feb 29 and sets the calendar to leap year free
 
+    Examples
+    ------------
+    Remove leap years from a dataset:
+    >>> ds.no_leaps()
+
     """
 
-    cdo_command = f"cdo -del29feb"
+    cdo_command = f"-del29feb"
 
-    run_this(cdo_command, self, output="ensemble")
+    self.cdo_command(cdo_command, ensemble=False)
 
 
 def set_gridtype(self, grid):
@@ -100,12 +187,12 @@ def set_gridtype(self, grid):
     ]:
         raise ValueError("Grid type supplies is not supported")
 
-    cdo_command = f"cdo -setgridtype,{grid}"
+    cdo_command = f"-setgridtype,{grid}"
 
-    run_this(cdo_command, self, output="ensemble")
+    self.cdo_command(cdo_command, ensemble=False)
 
 
-def assign_coords(self, lon_name=None, lat_name=None, time_name = None):
+def assign_coords(self, lon_name=None, lat_name=None, time_name=None):
     """
     Assign coordinates to variables
 
@@ -164,9 +251,7 @@ def assign_coords(self, lon_name=None, lat_name=None, time_name = None):
         nco_command = "ncatted "
 
         for vv in self.variables:
-            nco_command += (
-                "-a coordinates," + vv + ",c,c,'" + time_name + " ' "
-            )
+            nco_command += "-a coordinates," + vv + ",c,c,'" + time_name + " ' "
 
         target = temp_file("nc")
 
@@ -254,7 +339,6 @@ def delete_attributes(self, atts):
         atts = [atts]
 
     for i in atts:
-        i_dict = i
         nco_command += "-a " + i + ",global,d,, "
 
     target = ""
@@ -287,6 +371,11 @@ def as_type(self, x):
     x : dict
         A dictionary mapping variables to type. Values in dict must be one of 'int', 'float32' and 'float64'.
 
+    Examples
+    -------------
+    Change time to float64:
+    >>> ds.as_type({'time': 'float64'})
+
     """
 
     self.run()
@@ -311,7 +400,6 @@ def as_type(self, x):
         if x[xx] == "float64":
             x[xx] = "double"
 
-
     the_command = ""
     for xx in x:
         the_command += f" -s '{xx}={x[xx]}({xx})'"
@@ -329,6 +417,11 @@ def as_double(self, x):
     x : list
         A list of variable/dimensions you want to convert to floats
 
+    Examples
+    -------------
+    Change time to double:
+    >>> ds.as_double('time])
+
     """
 
     self.run()
@@ -344,6 +437,3 @@ def as_double(self, x):
 
     for xx in x:
         self.nco_command(f"ncap2 -s '{xx}=double({xx})'")
-
-
-
