@@ -1,9 +1,7 @@
-from nctoolkit.runthis import run_this
-
-
 def reduce_dims(self):
     """
-    Reduce dimensions of data
+    reduce_dims: Reduce dimensions of data
+
     This will remove any dimensions with only one value. For example, if only selecting
     one vertical level, the vertical dimension will be removed.
 
@@ -20,8 +18,8 @@ def reduce_dims(self):
     """
 
     if len(self.history) == len(self._hold_history):
-        cdo_command = "cdo --reduce_dim copy"
+        cdo_command = "--reduce_dim copy"
     else:
-        cdo_command = "cdo --reduce_dim"
+        cdo_command = "--reduce_dim"
 
-    run_this(cdo_command, self, output="ensemble")
+    self.cdo_command(cdo_command, ensemble=False)
