@@ -1,9 +1,6 @@
-from nctoolkit.runthis import run_this
-
-
 def resample_grid(self, factor=None):
     """
-    Resample the horizontal grid of a dataset
+    resample_grid: Resample the horizontal grid of a dataset
 
     Parameters
     -------------
@@ -26,5 +23,5 @@ def resample_grid(self, factor=None):
     if factor < 1:
         raise ValueError(f"{factor} is not a postive integer greater than 1")
 
-    cdo_command = f"cdo -samplegrid,{factor}"
-    run_this(cdo_command, self, output="ensemble")
+    cdo_command = f"-samplegrid,{factor}"
+    self.cdo_command(cdo_command, ensemble=False)
