@@ -1,8 +1,6 @@
-from nctoolkit.runthis import run_this
-
-def align(self, align = "right"):
+def align(self, align="right"):
     """
-    Method to align output time in temporal methods. 
+    Method to align output time in temporal methods.
 
     Parameters
     -------------
@@ -11,7 +9,7 @@ def align(self, align = "right"):
         Options are "left", "centre" and "right"
 
     """
-    
+
     if not isinstance(align, str):
         raise TypeError("Alignment must be str type")
 
@@ -33,7 +31,7 @@ def align(self, align = "right"):
     if "middle" in align:
         align = "midhigh"
 
-    if align not in ["first","middle","last", "midhigh"]:
+    if align not in ["first", "middle", "last", "midhigh"]:
         raise ValueError(f"{align} is not a valid align argument")
 
     new_align = f"--timestat_date {align}"
@@ -41,7 +39,8 @@ def align(self, align = "right"):
         if self._align != new_align:
             self.run()
 
-    self._align = new_align 
+    self._align = new_align
+
 
 def rollstat(self, window=None, stat="mean"):
     """Method to calculate the monthly statistic from a netCDF file"""
@@ -60,13 +59,13 @@ def rollstat(self, window=None, stat="mean"):
         raise ValueError(f"{window} is not a valid window!")
 
     # create the cdo call and run it
-    cdo_command = f"cdo -run{stat},{str(window)}"
-    run_this(cdo_command, self, output="ensemble")
+    cdo_command = f"-run{stat},{str(window)}"
+    self.cdo_command(cdo_command, ensemble=False)
 
 
-def rolling_mean(self, window=None, align = "right"):
+def rolling_mean(self, window=None, align="right"):
     """
-    Calculate a rolling mean based on a window
+    rolling_mean: Calculate a rolling mean based on a window
 
     Parameters
     -------------
@@ -76,7 +75,7 @@ def rolling_mean(self, window=None, align = "right"):
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
 
-         
+
 
     Examples
     ------------
@@ -90,9 +89,9 @@ def rolling_mean(self, window=None, align = "right"):
     rollstat(self, window=window, stat="mean")
 
 
-def rolling_min(self, window=None, align = "right"):
+def rolling_min(self, window=None, align="right"):
     """
-    Calculate a rolling minimum based on a window
+    rolling_min: Calculate a rolling minimum based on a window
 
     Parameters
     -------------
@@ -113,9 +112,9 @@ def rolling_min(self, window=None, align = "right"):
     rollstat(self, window=window, stat="min")
 
 
-def rolling_max(self, window=None, align = "right"):
+def rolling_max(self, window=None, align="right"):
     """
-    Calculate a rolling maximum based on a window
+    rolling_max: Calculate a rolling maximum based on a window
 
     Parameters
     -------------
@@ -135,9 +134,9 @@ def rolling_max(self, window=None, align = "right"):
     rollstat(self, window=window, stat="max")
 
 
-def rolling_range(self, window=None, align = "right"):
+def rolling_range(self, window=None, align="right"):
     """
-    Calculate a rolling range based on a window
+    rolling_range: Calculate a rolling range based on a window
 
     Parameters
     -------------
@@ -158,9 +157,9 @@ def rolling_range(self, window=None, align = "right"):
     rollstat(self, window=window, stat="range")
 
 
-def rolling_sum(self, window=None, align = "right"):
+def rolling_sum(self, window=None, align="right"):
     """
-    Calculate a rolling sum based on a window
+    rolling_sum: Calculate a rolling sum based on a window
 
     Parameters
     -------------
@@ -180,14 +179,15 @@ def rolling_sum(self, window=None, align = "right"):
     self.align(align)
     rollstat(self, window=window, stat="sum")
 
-def rolling_var(self, window=None, align = "right"):
+
+def rolling_var(self, window=None, align="right"):
     """
-    Calculate a rolling variance based on a window
+    rolling_var: Calculate a rolling variance based on a window
 
     Parameters
     -------------
     window: int
-        The size of the window for the calculation of the rolling sum
+        The size of the window for the calculation of the rolling variance
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
@@ -197,19 +197,21 @@ def rolling_var(self, window=None, align = "right"):
 
     If you wanted to calculate a rolling variance with the variance calculated over every 10 time steps, do the following:
 
-    >>> ds.rolling_sum(10)
+    >>> ds.rolling_var(10)
+
     """
     self.align(align)
     rollstat(self, window=window, stat="var")
 
-def rolling_stdev(self, window=None, align = "right"):
+
+def rolling_stdev(self, window=None, align="right"):
     """
-    Calculate a rolling standard deviation based on a window
+    rolling_stdev: Calculate a rolling standard deviation based on a window
 
     Parameters
     -------------
     window: int
-        The size of the window for the calculation of the rolling sum
+        The size of the window for the calculation of the rolling standard deviation
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
@@ -219,7 +221,8 @@ def rolling_stdev(self, window=None, align = "right"):
 
     If you wanted to calculate a rolling standard deviation with the standard deviation calculated over every 10 time steps, do the following:
 
-    >>> ds.rolling_sum(10)
+    >>> ds.rolling_stdev(10)
+
     """
     self.align(align)
     rollstat(self, window=window, stat="std")
