@@ -55,6 +55,7 @@ def fix_label(x):
     new_x = new_x.replace("/m**3", "m$^{-3}$")
     new_x = new_x.replace("/m^3", "m$^{-3}$")
     new_x = new_x.replace("/m^2", "m$^{-2}$")
+    new_x = new_x.replace("/yr", "year$^{-1}$")
     new_x = new_x.replace("/m2", "m$^{-2}$")
     new_x = new_x.replace("/d)", "d$^{-1}$)")
     new_x = new_x.replace("/s)", "s$^{-1}$)")
@@ -84,7 +85,6 @@ def pub_plot(
     norm=None,
     limits=None,
     projection="auto",
-    mid_point=None,
     coast="auto",
     scale="auto",
     grid=True,
@@ -93,6 +93,8 @@ def pub_plot(
     robust=False,
     out=None,
     breaks=None,
+    dpi = "figure",
+    font = None,
     **kwargs,
 ):
     """
@@ -120,10 +122,13 @@ def pub_plot(
         Character string with colour map to use. Set to None if you do not want to use a colour map.
     norm: str or matplotlib.colors norm
          Norm to use for colour bar
+    limits: list
+        List with [min, max] for colour bar limits.
+        Please note that if the colour scale passes through zero, the colour scale will be symmetrical around zero.
+        So the limits will be reset to the maximum absolute value of the data.
+
     projection: cartopy projection
         Cartopy projection to use.
-    mid_point: float
-        Mid-point to use for colour bar
     coast: str
         Set to "coarse", "low", "intermediate", "high" or "full" if you want to use GSHHS coastlines
     scale: str
@@ -137,6 +142,8 @@ def pub_plot(
         Output file name
     breaks : list
         List of breaks for the colour bar
+    dpi : int
+        DPI for output file. Default is "figure".
 
 
     *kwargs:
@@ -144,6 +151,7 @@ def pub_plot(
 
     -------------
     """
+    mid_point = None
 
     relief = False
     # check if relief is in kwargs
@@ -203,6 +211,8 @@ def pub_plot(
         fig = None
     if "gs" not in kwargs.keys():
         gs = None
+    if "dpi" not in kwargs.keys():
+        dpi = "figure"
 
     if "quiver" in kwargs:
         quiver = True
@@ -220,6 +230,9 @@ def pub_plot(
         if kk == "fig":
             fig = kwargs[kk]
             fixed = True
+        if kk == "dpi":
+            fixed = True
+            dpi = kwargs[kk]
 
         if kk == "gs":
             gs = kwargs[kk]
@@ -239,10 +252,10 @@ def pub_plot(
                 norm = kwargs[kk]
                 fixed = True
 
-        if kk.lower().startswith("mid") and kk.lower().endswith("nt"):
-            if mid_point is None:
-                mid_point = kwargs[kk]
-                fixed = True
+        # if kk.lower().startswith("mid") and kk.lower().endswith("nt"):
+        #    if mid_point is None:
+        #        mid_point = kwargs[kk]
+        #        fixed = True
 
         if kk.lower().startswith("proj"):
             if projection == "auto":
@@ -349,8 +362,8 @@ def pub_plot(
 
     # set globe to True if lon lat spread is big enough
 
-    if np.max(lon) - np.min(lon) > 358:
-        if np.max(lat) - np.min(lat) > 176:
+    if np.max(lon) - np.min(lon) > 340:
+        if np.max(lat) - np.min(lat) > 160:
             globe = True
 
     # proj=ccrs.LambertConformal(central_longitude=np.mean(lon), central_latitude=np.mean(lat), false_easting=0.0, false_northing=0.0,  cutoff=38)
@@ -377,6 +390,13 @@ def pub_plot(
 
     if projection == None:
         proj = ccrs.PlateCarree()
+
+    if isinstance(projection, ccrs.NorthPolarStereo) and size == "auto":
+        size = [8,8]
+    if isinstance(projection, ccrs.SouthPolarStereo) and size == "auto":
+        size = [8,8]
+    if isinstance(projection, ccrs.AzimuthalEquidistant) and size == "auto":
+        size  = [12, 8]
     if size == "auto":
         size = [10, 10]
         if proj == ccrs.PlateCarree() or globe:
@@ -459,24 +479,6 @@ def pub_plot(
             if r_max is not None:
                 max_value = np.nanpercentile(np.ma.filled(values, np.nan), r_max)
 
-        if limits is not None and colours == "auto":
-            try:
-                if limits[0] < 0 and limits[1] > 0:
-                    min_value = limits[0]
-                    max_value = limits[1]
-                    if mid_point is None:
-                        mid_point = 0
-                    if colours == "auto":
-                        colours = "RdBu_r"
-                        if land_auto and land is not None:
-                            if land != "auto":
-                                land = "grey"
-            except:
-                pass
-
-        if colours == "auto":
-            colours = "viridis"
-
     # generate the figure and the axes where to plot the map. When the axes are generated (either with add_subplot or add_axes, or any other way), the projection to be used need to be specified
     # the axes generated show the entire globe up to the cutoff latitude
     if fig is None:
@@ -514,15 +516,48 @@ def pub_plot(
             if r_max is not None:
                 vmax = np.nanpercentile(np.ma.filled(values, np.nan), r_max)
 
-        if robust and limits is None:
+        if robust:
             vmin = np.nanpercentile(np.ma.filled(values, np.nan), 2)
             vmax = np.nanpercentile(np.ma.filled(values, np.nan), 98)
+            limits = [0,0]
+            limits[0] = vmin
+            limits[1] = vmax
+
         if limits is None:
             if r_min is not None:
                 vmin = np.nanpercentile(np.ma.filled(values, np.nan), r_min)
             if r_max is not None:
                 vmax = np.nanpercentile(np.ma.filled(values, np.nan), r_max)
 
+        if mid_point is None:
+            if vmin < 0 and vmax > 0:
+                mid_point = 0
+                limits[0] = vmin
+                limits[1] = vmax
+                try:
+                    if limits[0] < 0 and limits[1] > 0:
+                        min_value = limits[0]
+                        max_value = limits[1]
+                        if mid_point is None:
+                            mid_point = 0
+                        if colours == "auto":
+                            colours = "RdBu_r"
+                            if land_auto and land is not None:
+                                if land != "auto":
+                                    land = "grey"
+                except:
+                    pass
+
+        if mid_point is not None and limits is not None:
+            try:
+                if colours == "auto":
+                    colours = "RdBu_r"
+                    if land_auto and land is not None:
+                        if land != "auto":
+                            land = "grey"
+            except:
+                pass
+
         if mid_point is not None:
             val_min = values.min()
             val_max = values.max()
@@ -550,6 +585,32 @@ def pub_plot(
             vmin = None
             vmax = None
 
+        if limits is not None and colours == "auto":
+            try:
+                if limits[0] < 0 and limits[1] > 0:
+                    min_value = limits[0]
+                    max_value = limits[1]
+                    if mid_point is None:
+                        mid_point = 0
+                    if colours == "auto":
+                        colours = "RdBu_r"
+                        if land_auto and land is not None:
+                            if land != "auto":
+                                land = "grey"
+                else:
+                    colours = "viridis"
+            except:
+                pass
+
+        if colours == "auto":
+            colours = "viridis"
+
+        if norm is None:
+            if vmin < 0 and vmax > 0:
+                # ensure vmin and vmax are symmetric
+                vmax = max(abs(vmin), abs(vmax))
+                vmin = -vmax
+
         im = ax.pcolormesh(
             lon,
             lat,
@@ -636,10 +697,34 @@ def pub_plot(
         else:
             fraction = 0.046 * size[1] / size[0]
 
-        if l_location == "bottom":
-            cb = plt.colorbar(im, fraction=fraction, pad=0.04, location=l_location)
+        min_value = np.min(values)
+        max_value = np.max(values)
+
+        min_arrow = False
+
+        if vmin is not None:
+            if vmin > min_value:
+                min_arrow = True
+        max_arrow = False
+        if vmax is not None:
+            if vmax < max_value:
+                max_arrow = True
+        
+        if min_arrow and max_arrow:
+            extend = "both"
         else:
-            cb = plt.colorbar(im, fraction=fraction, pad=0.04)
+            if min_arrow:
+                extend = "min"
+            else:
+                if max_arrow:
+                    extend = "max"
+                else:
+                    extend = "neither"
+
+        if l_location == "bottom":
+            cb = plt.colorbar(im, fraction=fraction, pad=0.04, location=l_location, extend = extend)
+        else:
+            cb = plt.colorbar(im, fraction=fraction, pad=0.04, extend = extend)
 
         # add breaks to colorbar cb
         if breaks is not None:
@@ -694,11 +779,12 @@ def pub_plot(
                     "Unable to parse legend from dataset contents. Check long names and units"
                 )
                 label = ""
+            # label may need some slight tidying
 
             if l_location == "bottom":
                 label = "\n".join(wrap(label, 120))
             else:
-                label = "\n".join(wrap(label, 50))
+                label = "\n".join(wrap(label, 30))
 
             if l_location == "bottom":
                 cbax.set_xlabel(label)
@@ -707,10 +793,18 @@ def pub_plot(
 
         if legend_position is None:
             cb.remove()
+    
+    if font is not None:
+        for text in [ax.title, ax.xaxis.label, ax.yaxis.label, cb.ax.yaxis.label, cb.ax.xaxis.label]:
+            text.set_fontsize(font)
+        for item in ([ax.title, ax.xaxis.label, ax.yaxis.label, cb.ax.yaxis.label, cb.ax.xaxis.label] +
+                 ax.get_xticklabels() + ax.get_yticklabels()):
+            item.set_fontsize(font)
 
     if out is not None:
         print("saving as file")
-        plt.savefig(out)
+        plt.savefig(out, dpi = dpi)
+        plt.savefig(out, dpi = dpi)
 
 
 def quiver_plot(ds, u=None, v=None, **kwargs):
@@ -767,4 +861,4 @@ def quiver_plot(ds, u=None, v=None, **kwargs):
     if v not in vars:
         raise ValueError("v not in dataset")
 
-    pub_plot(ds, quiver=True, u=u, v=v, **kwargs)
+    pub_plot(ds, quiver=True, u=u, v=v, **kwargs)
\ No newline at end of file
