@@ -3,11 +3,14 @@ import copy
 from nctoolkit.flatten import str_flatten
 from nctoolkit.runthis import run_nco
 from nctoolkit.temp_file import temp_file
+from nctoolkit.session import get_safe
+from nctoolkit.session import remove_safe
 
 
 def strip_variables(self, vars=None):
     """
-    Remove any variables, such as bnds etc., from variables.
+    strip_variables: Remove any variables, such as bnds etc., from variables.
+
     This should probably only be done at the end of a processing chain before converting to a dataframe etc., as it is stripping away critical info for netCDF operations.
 
     Parameters
@@ -20,7 +23,6 @@ def strip_variables(self, vars=None):
     self.run()
 
     if vars is not None:
-
         if isinstance(vars, str):
             vars = [vars]
 
@@ -31,20 +33,23 @@ def strip_variables(self, vars=None):
         command = f"ncks -C -v {str_flatten(vars)}"
 
     else:
-
         command = f"ncks -C -v {str_flatten(self.variables)}"
 
     new_files = []
     new_commands = []
 
-    for ff in self:
+    delete_these = []
 
+    for ff in self:
         target = temp_file(".nc")
 
         the_command = f"{command} {ff} {target}"
 
         target = run_nco(the_command, target=target)
 
+        if target in get_safe():
+            delete_these.append(target)
+
         new_files.append(target)
         new_commands.append(the_command)
 
@@ -53,4 +58,7 @@ def strip_variables(self, vars=None):
     self.history.append(command)
     self._hold_history = copy.deepcopy(self.history)
 
+    for ff in delete_these:
+        remove_safe(ff)
+
     self.disk_clean()
