@@ -1,23 +1,39 @@
 import copy
 
 from nctoolkit.cleanup import cleanup
-from nctoolkit.runthis import run_this, run_cdo, tidy_command
+from nctoolkit.runthis import run_cdo, tidy_command
 from nctoolkit.temporals import *
 from nctoolkit.temp_file import temp_file
 from nctoolkit.session import remove_safe
-import warnings
 
 
-def time_stat(self, stat="mean", over="time"):
+def time_stat(self, stat="mean", over="time", window = None):
     """Method to calculate a stat over all time steps"""
     # create cdo command and run it
 
+    # check if window is an int
+    if window is not None:
+        if isinstance(window, int):
+
+            ## check if over is time 
+            if over != "time":
+                raise ValueError("You cannot supply a window if you are not grouping over all time periods")
+
+            cdo_command = f"-timsel{stat},{window}"
+            self.cdo_command(cdo_command, ensemble=False)
+            return None
+        else:
+            raise ValueError("Window must be an integer")
+    
+
+
+
     if len(self) == 0:
         raise ValueError("Failure due to empty dataset!")
 
     if over == "time":
-        cdo_command = f"cdo -tim{stat}"
-        run_this(cdo_command, self, output="ensemble")
+        cdo_command = f"-tim{stat}"
+        self.cdo_command(cdo_command, ensemble=False)
         return None
 
     if stat not in ["mean", "sum", "min", "max", "range", "var", "cumsum", "std"]:
@@ -67,7 +83,7 @@ def time_stat(self, stat="mean", over="time"):
         yhourstat(self, stat=stat)
         return None
 
-    if over == ["day", "hour",  "year"]:
+    if over == ["day", "hour", "year"]:
         run = True
         hourstat(self, stat=stat)
         return None
@@ -118,24 +134,42 @@ def time_stat(self, stat="mean", over="time"):
         raise ValueError(f"Grouping {over} is currently not supported!")
 
 
-
-def tsum(self, over="time", align = "right"):
+def tsum(self, over="time", align="right", window = None):
     """
-    Calculate the temporal sum of all variables
+    tsum: Calculate the temporal sum of all variables.
 
     Parameters
     -------------
-    align = str
+    align : str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    over : str or list
+        Time periods to count the sum over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R,
+        so you can supply combinations of these to get the sum over each year, month or day.
+    window : int
+        This determines the number of time steps to sum over, on a non-rolling basis. 
+        This is useful if you need to calculate the sum every 5 days, for example.
+
+    Examples
+    ------------
+    If you want to calculate sum over all time steps. Do the following:
+    >>> ds.tsum()
+    If you want to calculate the sum over each year:
+    >>> ds.tsum(over="year")
+    If you want to calculate the sum over each month. This will add up all data in each month across all years not within each year.
+    >>> ds.tsum(over="month")
+    If you want to calculate the sum over each day. This will add up all data in each day across all years not within each year.
+    >>> ds.tsum(over="day")
+
     """
     self.align(align)
-    time_stat(self, stat="sum", over=over)
+    time_stat(self, stat="sum", over=over, window = window)
 
 
-def na_count(self, over="time", align = "right"):
+def na_count(self, over="time", align="right", window = None):
     """
-    Calculate the number of missing values
+    na_count: Calculate the number of missing values.
 
     Parameters
     -------------
@@ -145,6 +179,16 @@ def na_count(self, over="time", align = "right"):
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window : int
+        This determines the number of time steps to calculate, on a non-rolling basis. 
+        This is useful if you need to calculate the sum every 5 days, for example.
+
+    Examples
+    ------------
+    If you want to calculate the number of missing values over all time steps. Do the following:
+    >>> ds.na_count()
+    If you want to calculate the number of missing values in each year:
+    >>> ds.na_count(over="year")
     """
     self.align(align)
 
@@ -153,12 +197,12 @@ def na_count(self, over="time", align = "right"):
     for vv in self.variables:
         self.cdo_command(f"-aexpr,'{vv}=isMissval({vv})'")
 
-    self.tsum(over=over)
+    self.tsum(over=over, window = window)
 
 
-def na_frac(self, over="time", align = "right"):
+def na_frac(self, over="time", align="right", window = None):
     """
-    Calculate the number of missing values
+    na_frac: Calculate the fraction of missing values in each grid cell across all time steps.
 
     Parameters
     -------------
@@ -168,6 +212,17 @@ def na_frac(self, over="time", align = "right"):
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window: int
+        This determines the number of time steps to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the fraction for every non-overlapping 5 day periods, for example.
+
+    Examples
+    ------------
+    If you want to calculate the fraction of missing values over all time steps. Do the following:
+    >>> ds.na_frac()
+    If you want to calculate the fraction of missing values in each year:
+    >>> ds.na_frac(over="year")
+
     """
     self.align(align)
 
@@ -176,21 +231,27 @@ def na_frac(self, over="time", align = "right"):
     for vv in self.variables:
         self.cdo_command(f"-aexpr,'{vv}=isMissval({vv})'")
 
-    self.tmean(over=over)
+    self.tmean(over=over, window = window)
 
 
-def tmean(self, over="time", align = "right"):
+def tmean(self, over="time", align="right", window = None):
     """
-    Calculate the temporal mean of all variables
+    tmean: Calculate the temporal mean of all variables.
+
+    Useful for: monthly mean, annual/yearly mean, seasonal mean, daily mean, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R.
 
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window: int
+        This determines the number of time steps to calculate the mean over to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the mean every 5 days, for example.
 
     Examples
     ------------
@@ -202,10 +263,6 @@ def tmean(self, over="time", align = "right"):
 
         >>> ds.tmean("year")
 
-    If you want to calculate the mean for each month in a dataset, do this:
-
-        >>> ds.tmean("month")
-
     If you want to calculate the mean for each month in each year in a dataset, do this:
 
         >>> ds.tmean(["year", "month"])
@@ -221,22 +278,28 @@ def tmean(self, over="time", align = "right"):
 
 
     """
-    self.align(align = align)
-    time_stat(self, stat="mean", over=over)
+    self.align(align=align)
+    time_stat(self, stat="mean", over=over, window = window)
 
 
-def tmin(self, over="time", align = "right"):
+def tmin(self, over="time", align="right", window = None):
     """
-    Calculate the temporal minimum of all variables
+    tmin: Calculate the temporal minimum of all variables.
+
+    Useful for: monthly minimum, annual/yearly minimum, seasonal minimum, daily minimum, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
 
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window: int
+        This determines the number of time steps to calculate the minimum over to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the minimum every 5 days, for example.
 
     Examples
     ------------
@@ -266,21 +329,27 @@ def tmin(self, over="time", align = "right"):
         >>> ds.tmin( "day")
 
     """
-    self.align(align = align)
-    time_stat(self, stat="min", over=over)
+    self.align(align=align)
+    time_stat(self, stat="min", over=over, window = window)
 
 
-def tmax(self, over="time", align = "right"):
+def tmax(self, over="time", align="right", window = None):
     """
-    Calculate the temporal maximum of all variables
+    tmax: Calculate the temporal maximum of all variables.
+
+    Useful for: monthly maximum, annual/yearly maximum, seasonal maximum, daily maximum, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
-    align = str
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
+    align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window: int
+        This determines the number of time steps to calculate the maximum over to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the maximum every 5 days, for example.
 
     Examples
     ------------
@@ -309,18 +378,21 @@ def tmax(self, over="time", align = "right"):
 
         >>> ds.tmax( "day")
     """
-    self.align(align = align)
-    time_stat(self, stat="max", over=over)
+    self.align(align=align)
+    time_stat(self, stat="max", over=over, window = window)
 
 
-def tmedian(self, over="time", align = "right"):
+def tmedian(self, over="time", align="right"):
     """
-    Calculate the temporal median of all variables
+    tmedian: Calculate the temporal median of all variables.
+
+    Useful for: monthly median, annual/yearly median, seasonal median, daily median, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
@@ -352,21 +424,26 @@ def tmedian(self, over="time", align = "right"):
 
         >>> ds.tmedian( "day")
     """
-    self.align(align = align)
+    self.align(align=align)
     self.tpercentile(p=50, over=over)
 
 
-def trange(self, over="time", align = "right"):
+def trange(self, over="time", align="right", window = None):
     """
-    Calculate the temporal range of all variables
+    trange: Calculate the temporal range of all variables
+    Useful for: monthly range, annual/yearly range, seasonal range, daily range, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window: int
+        This determines the number of time steps to calculate the range over to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the range every 5 days, for example.
 
     Examples
     ------------
@@ -396,22 +473,26 @@ def trange(self, over="time", align = "right"):
         >>> ds.trange( "day")
 
     """
-    self.align(align = align)
-    time_stat(self, stat="range", over=over)
+    self.align(align=align)
+    time_stat(self, stat="range", over=over, window = window)
 
 
-def tvar(self, over="time", align = "right"):
+def tvar(self, over="time", align="right", window = None):
     """
-    Calculate the temporal variance of all variables
+    tvar: Calculate the temporal variance of all variables
+    Useful for: monthly variance, annual/yearly variance, seasonal variance, daily variance, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
-
+    window: int
+        This determines the number of time steps to calculate the variance over to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the variance every 5 days, for example.
 
     Examples
     ------------
@@ -440,21 +521,26 @@ def tvar(self, over="time", align = "right"):
 
         >>> ds.tvar( "day")
     """
-    self.align(align = align)
-    time_stat(self, stat="var", over=over)
+    self.align(align=align)
+    time_stat(self, stat="var", over=over, window = window)
 
 
-def tstdev(self, over="time", align = "right"):
+def tstdev(self, over="time", align="right", window = None):
     """
-    Calculate the temporal standard deviation of all variables
+    tstdev: Calculate the temporal standard deviation of all variables
+    Useful for: monthly standard deviation, annual/yearly standard deviation, seasonal standard deviation, daily standard deviation, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     over: str or list
         Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
+    window: int
+        This determines the number of time steps to calculate the standard deviation over to calculate over, on a non-rolling basis. 
+        This is useful if you need to calculate the standard deviation every 5 days, for example.
 
 
     Examples
@@ -484,13 +570,13 @@ def tstdev(self, over="time", align = "right"):
 
         >>> ds.tstdev("day")
     """
-    self.align(align = align)
-    time_stat(self, stat="std", over=over)
+    self.align(align=align)
+    time_stat(self, stat="std", over=over, window = window)
 
 
-def tcumsum(self, align = "right"):
+def tcumsum(self, align="right"):
     """
-    Calculate the temporal cumulative sum of all variables
+    tcumsum: Calculate the temporal cumulative sum of all variables
 
     Parameters
     -------------
@@ -510,14 +596,18 @@ def tcumsum(self, align = "right"):
     time_stat(self, stat="cumsum")
 
 
-def tpercentile(self, p=None, over="time", align = "right"):
+def tpercentile(self, p=None, over="time", align="right"):
     """
-    Calculate the temporal percentile of all variables
+    tpercentile: Calculate the temporal percentile of all variables
+    Useful for monthly percentile, annual/yearly percentile, seasonal percentile, daily percentile, daily climatology, monthly climatology, seasonal climatology
 
     Parameters
     -------------
     p: float or int
         Percentile to calculate
+    over: str or list
+        Time periods to average over. Options are 'year', 'month', 'day'.
+        This operates in a similar way to the groupby method in pandas or the tidyverse in R, with over acting as the grouping.
     align: str
         This determines whether the output time is at the left, centre or right hand side of the time window.
         Options are "left", "centre" and "right"
@@ -532,6 +622,10 @@ def tpercentile(self, p=None, over="time", align = "right"):
 
         >>> ds.tpercentile(20)
 
+    If you want to calculate the 20th percentile for each year in a dataset, do this:
+
+        >>> ds.tpercentile(p= 20, over = "year")
+
     """
     self.align(align=align)
 
@@ -625,8 +719,8 @@ def tpercentile(self, p=None, over="time", align = "right"):
         target = temp_file("nc")
 
         cdo_command = (
-                perc_term
-            #"cdo -timpctl,"
+            perc_term
+            # "cdo -timpctl,"
             + str(p)
             + " "
             + ff
