@@ -8,7 +8,8 @@ from nctoolkit.session import remove_safe
 
 def to_nc(self, out, zip=True, overwrite=False, **kwargs):
     """
-    Save a dataset to a named file
+    to_nc: Save a dataset to a named file.
+
     This will only work with single file datasets.
 
     Parameters
@@ -44,6 +45,10 @@ def to_nc(self, out, zip=True, overwrite=False, **kwargs):
 
     """
 
+    # out may have ~ in it, which may need fixing
+
+    out = os.path.expanduser(out)
+
     if len(self) == 0:
         raise ValueError("You cannot save an empty dataset!")
 
@@ -52,7 +57,6 @@ def to_nc(self, out, zip=True, overwrite=False, **kwargs):
         if os.path.exists(out_dir) is False:
             raise ValueError(f"{out_dir} does not exist!")
 
-
     # Figure out if it is possible to write the file, i.e. if a dataset is still an
     # ensemble, you cannot write.
     ff = copy.deepcopy(self.current)
@@ -65,21 +69,18 @@ def to_nc(self, out, zip=True, overwrite=False, **kwargs):
         write = True
 
     if write is False:
-        raise ValueError("You cannot save multiple files!")
+        raise ValueError("You cannot save multiple files! Please use a method such as merge to create one file.")
 
     if (os.path.exists(out)) and (overwrite is True):
         if len(self) > 1:
             self.run()
 
-
     # Check if outfile exists and overwrite is set to False
     # This should maybe be a warning, not an error
     if (os.path.exists(out)) and (overwrite is False):
         raise ValueError("The out file exists and overwrite is set to false")
 
-
     if len(kwargs) > 0:
-
         self1 = self.copy()
 
         self1.subset(**kwargs)
@@ -87,63 +88,60 @@ def to_nc(self, out, zip=True, overwrite=False, **kwargs):
         self1.run()
         ff = copy.deepcopy(self1.current)
 
+        if zip:
+            cdo_command = f"cdo -z zip_9 copy {ff[0]} {out}"
+            run_cdo(
+                cdo_command,
+                target=out,
+                overwrite=overwrite,
+                precision=self._precision,
+            )
 
-        if len(self1.history) == len(self1._hold_history):
-            if zip:
-                cdo_command = f"cdo -z zip_9 copy {ff[0]} {out}"
-                run_cdo(
-                    cdo_command, target=out, overwrite=overwrite, precision=self._precision
-                )
-
-                self1.history.append(cdo_command)
-                self1._hold_history = copy.deepcopy(self1.history)
-                self1.current = out
-                remove_safe(out)
-
-            else:
-                cdo_command = f"cdo copy {ff[0]} {out}"
-                run_cdo(
-                    cdo_command, target=out, overwrite=overwrite, precision=self1._precision
-                )
-                self1.history.append(cdo_command)
-                self1._hold_history = copy.deepcopy(self1.history)
-
-                self1.current = out
-                remove_safe(out)
+            self1.history.append(cdo_command)
+            self1._hold_history = copy.deepcopy(self1.history)
+            self1.current = out
+            remove_safe(out)
 
         else:
-            if zip:
-                cdo_command = "cdo -z zip_9 "
-            else:
-                cdo_command = "cdo "
+            cdo_command = f"cdo copy {ff[0]} {out}"
+            run_cdo(
+                cdo_command,
+                target=out,
+                overwrite=overwrite,
+                precision=self1._precision,
+            )
+            self1.history.append(cdo_command)
+            self1._hold_history = copy.deepcopy(self1.history)
 
-            self1._execute = True
-
-            run_this(cdo_command, self1, out_file=out)
-            self1._execute = False
+            self1.current = out
+            remove_safe(out)
 
         if os.path.exists(out) is False:
             raise ValueError("File zipping was not successful")
     else:
-
         ff = copy.deepcopy(self.current)
 
         if len(self.history) == len(self._hold_history):
             if zip:
                 cdo_command = f"cdo -z zip_9 copy {ff[0]} {out}"
                 run_cdo(
-                    cdo_command, target=out, overwrite=overwrite, precision=self._precision
+                    cdo_command,
+                    target=out,
+                    overwrite=overwrite,
+                    precision=self._precision,
                 )
                 self.history.append(cdo_command)
                 self._hold_history = copy.deepcopy(self.history)
                 self.current = out
                 remove_safe(out)
 
-
             else:
                 cdo_command = f"cdo copy {ff[0]} {out}"
                 run_cdo(
-                    cdo_command, target=out, overwrite=overwrite, precision=self._precision
+                    cdo_command,
+                    target=out,
+                    overwrite=overwrite,
+                    precision=self._precision,
                 )
                 self.history.append(cdo_command)
                 self._hold_history = copy.deepcopy(self.history)
@@ -165,5 +163,4 @@ def to_nc(self, out, zip=True, overwrite=False, **kwargs):
         if os.path.exists(out) is False:
             raise ValueError("File zipping was not successful")
 
-
     cleanup()
