@@ -32,32 +32,44 @@ def generate_level_bounds(self):
 
 
 
-def bottom(self):
+def bottom(self, choice = "level"):
     """
-    bottom: Extract the bottom level from a dataset
+    bottom: Extract the bottom level or value from a dataset
 
-    This extracts the bottom level from each netCDF file. Please note that for
+    This extracts the bottom level or value from each netCDF file. Please note that for
     ensembles, it uses the first file to derive the index of the bottom level.
 
-    You may need to double check that the bottom vertical level is the sea 'bottom' etc., as this is not always the case.
-
-    Use bottom_mask for files when the bottom cell in netCDF files do not represent
-    the actual bottom.
+    Parameters
+    -------------
+    choice : str
+        Set to 'level' to extract the bottom level, or 'value' to extract the bottommost non-missing value. 
 
     Examples
     ------------
 
     If you wanted to extract the bottom vertical level of a dataset, do the following:
 
-    >>> ds.bottom()
+    >>> ds.bottom(choice = 'level')
 
     This method is most useful for things like oceanic model data, where the bottom cell corresponds to the bottom of the ocean.
 
+    If you wanted to find the deepest non-missing value in a dataset, you would do the following:
+
+    >>> ds.bottom(choice = 'value')
+
     """
 
     # extract the number of the bottom level
     # Use the first file for an ensemble
     # pull the cdo command together, then run it or store it
+
+    if choice not in ["level", "value"]:
+        raise ValueError("Choice must be either 'level' or 'value'")
+
+    if choice == "value":
+        self.cdo_command("bottomvalue")
+        return None
+
     if len(self) > 1:
         ff = self.current[0]
         warnings.warn(
@@ -164,21 +176,23 @@ def vertical_interp(
             if surface is None:
                 raise ValueError("Please provide surface")
 
-    if fixed is None:
-        if thickness is None:
-            raise ValueError("You must provide the fixed arg")
-
     if depths is None:
         if fixed is False:
             if thickness is None:
-                raise ValueError("Please provide thickness")
+                raise ValueError("Please provide thickness or depths")
+
+    if thickness is not None:
+        fixed = False
+
+    if depths is not None:
+        fixed = False
 
     if not isinstance(fixed, bool):
         if thickness is None:
             raise TypeError("fixed must be a bool")
 
-    if thickness is not None:
-        fixed = False
+    if isinstance(levels, (int, float)):
+        levels = [levels]
 
     if fixed is False:
         self.to_zlevels(
@@ -191,8 +205,6 @@ def vertical_interp(
 
     # first a quick fix for the case when there is only one vertical depth
 
-    if isinstance(levels, (int, float)):
-        levels = [levels]
 
     # levels = [float(x) for x in levels]
 
